# coding: utf-8

# flake8: noqa
"""
    zrok

    zrok client access

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from zrok_api.models.access201_response import Access201Response
from zrok_api.models.access_request import AccessRequest
from zrok_api.models.add_frontend_grant_request import AddFrontendGrantRequest
from zrok_api.models.add_organization_member_request import AddOrganizationMemberRequest
from zrok_api.models.auth_user import AuthUser
from zrok_api.models.change_password_request import ChangePasswordRequest
from zrok_api.models.client_version_check_request import ClientVersionCheckRequest
from zrok_api.models.configuration import Configuration
from zrok_api.models.create_frontend201_response import CreateFrontend201Response
from zrok_api.models.create_frontend_request import CreateFrontendRequest
from zrok_api.models.create_identity201_response import CreateIdentity201Response
from zrok_api.models.create_identity_request import CreateIdentityRequest
from zrok_api.models.create_organization201_response import CreateOrganization201Response
from zrok_api.models.create_organization_request import CreateOrganizationRequest
from zrok_api.models.delete_identity_request import DeleteIdentityRequest
from zrok_api.models.disable_request import DisableRequest
from zrok_api.models.enable_request import EnableRequest
from zrok_api.models.enroll200_response import Enroll200Response
from zrok_api.models.enroll_request import EnrollRequest
from zrok_api.models.environment import Environment
from zrok_api.models.environment_and_resources import EnvironmentAndResources
from zrok_api.models.frontend import Frontend
from zrok_api.models.get_sparklines200_response import GetSparklines200Response
from zrok_api.models.get_sparklines_request import GetSparklinesRequest
from zrok_api.models.invite_request import InviteRequest
from zrok_api.models.invite_token_generate_request import InviteTokenGenerateRequest
from zrok_api.models.list_frontends200_response_inner import ListFrontends200ResponseInner
from zrok_api.models.list_memberships200_response import ListMemberships200Response
from zrok_api.models.list_memberships200_response_memberships_inner import ListMemberships200ResponseMembershipsInner
from zrok_api.models.list_organization_members200_response import ListOrganizationMembers200Response
from zrok_api.models.list_organization_members200_response_members_inner import ListOrganizationMembers200ResponseMembersInner
from zrok_api.models.list_organizations200_response import ListOrganizations200Response
from zrok_api.models.list_organizations200_response_organizations_inner import ListOrganizations200ResponseOrganizationsInner
from zrok_api.models.list_public_frontends_for_account200_response import ListPublicFrontendsForAccount200Response
from zrok_api.models.list_public_frontends_for_account200_response_public_frontends_inner import ListPublicFrontendsForAccount200ResponsePublicFrontendsInner
from zrok_api.models.login_request import LoginRequest
from zrok_api.models.metrics import Metrics
from zrok_api.models.metrics_sample import MetricsSample
from zrok_api.models.overview import Overview
from zrok_api.models.ping200_response import Ping200Response
from zrok_api.models.principal import Principal
from zrok_api.models.regenerate_account_token200_response import RegenerateAccountToken200Response
from zrok_api.models.regenerate_account_token_request import RegenerateAccountTokenRequest
from zrok_api.models.register_request import RegisterRequest
from zrok_api.models.remote_access_request import RemoteAccessRequest
from zrok_api.models.remote_share200_response import RemoteShare200Response
from zrok_api.models.remote_share_request import RemoteShareRequest
from zrok_api.models.remote_status200_response import RemoteStatus200Response
from zrok_api.models.remote_status200_response_accesses_inner import RemoteStatus200ResponseAccessesInner
from zrok_api.models.remote_status200_response_shares_inner import RemoteStatus200ResponseSharesInner
from zrok_api.models.remote_unaccess_request import RemoteUnaccessRequest
from zrok_api.models.remote_unshare_request import RemoteUnshareRequest
from zrok_api.models.remove_organization_member_request import RemoveOrganizationMemberRequest
from zrok_api.models.reset_password_request import ResetPasswordRequest
from zrok_api.models.share import Share
from zrok_api.models.share_http_healthcheck200_response import ShareHttpHealthcheck200Response
from zrok_api.models.share_http_healthcheck_request import ShareHttpHealthcheckRequest
from zrok_api.models.share_request import ShareRequest
from zrok_api.models.share_response import ShareResponse
from zrok_api.models.spark_data_sample import SparkDataSample
from zrok_api.models.unaccess_request import UnaccessRequest
from zrok_api.models.unshare_request import UnshareRequest
from zrok_api.models.update_access_request import UpdateAccessRequest
from zrok_api.models.update_frontend_request import UpdateFrontendRequest
from zrok_api.models.update_share_request import UpdateShareRequest
from zrok_api.models.verify200_response import Verify200Response
from zrok_api.models.verify_request import VerifyRequest
from zrok_api.models.version_inventory200_response import VersionInventory200Response



from setuptools import setup, find_packages  # noqa: H301
import os
import versioneer
from pathlib import Path
import re

NAME = os.getenv('ZROK_PY_NAME', "zrok")
VERSION = "1.0.0"

OVERRIDES = {
    # Override specific packages with version constraints different from the generated requirements.txt
    "openziti": "openziti >= 1.0.0",
    # urllib3 2.1.0 introduced breaking changes that are implemented by openapi-generator 7.12.0
    "urllib3": "urllib3 >= 2.1.0",
}


# Parse the generated requirements.txt
def parse_requirements(filename):
    requirements = []
    if not Path(filename).exists():
        return requirements

    with open(filename, 'r') as f:
        for line in f:
            line = line.strip()
            if not line or line.startswith('#'):
                continue

            # Extract package name (everything before any version specifier)
            package_name = re.split(r'[<>=~]', line)[0].strip()

            # If we have an override for this package, skip it
            if package_name in OVERRIDES:
                continue

            requirements.append(line)

    return requirements


# Combine requirements from requirements.txt and overrides
requirements_file = Path(__file__).parent / "requirements.txt"
REQUIRES = parse_requirements(requirements_file) + list(OVERRIDES.values())

setup(
    name=NAME,
    cmdclass=versioneer.get_cmdclass(dict()),
    version=versioneer.get_version(),
    description="zrok",
    author_email="",
    url="",
    keywords=["zrok"],
    install_requires=REQUIRES,
    python_requires='>3.10.0',
    packages=find_packages(),
    include_package_data=True,
    package_data={
        '': ['requirements.txt'],  # Include the generated requirements.txt in the package
    },
    long_description="""\
    Geo-scale, next-generation peer-to-peer sharing platform built on top of OpenZiti.
    """
)



# coding: utf-8

"""
    zrok

    zrok client access

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from zrok_api.api.metadata_api import MetadataApi


class TestMetadataApi(unittest.TestCase):
    """MetadataApi unit test stubs"""

    def setUp(self) -> None:
        self.api = MetadataApi()

    def tearDown(self) -> None:
        pass

    def test_client_version_check(self) -> None:
        """Test case for client_version_check

        """
        pass

    def test_configuration(self) -> None:
        """Test case for configuration

        """
        pass

    def test_get_account_detail(self) -> None:
        """Test case for get_account_detail

        """
        pass

    def test_get_account_metrics(self) -> None:
        """Test case for get_account_metrics

        """
        pass

    def test_get_environment_detail(self) -> None:
        """Test case for get_environment_detail

        """
        pass

    def test_get_environment_metrics(self) -> None:
        """Test case for get_environment_metrics

        """
        pass

    def test_get_frontend_detail(self) -> None:
        """Test case for get_frontend_detail

        """
        pass

    def test_get_share_detail(self) -> None:
        """Test case for get_share_detail

        """
        pass

    def test_get_share_metrics(self) -> None:
        """Test case for get_share_metrics

        """
        pass

    def test_get_sparklines(self) -> None:
        """Test case for get_sparklines

        """
        pass

    def test_list_memberships(self) -> None:
        """Test case for list_memberships

        """
        pass

    def test_list_org_members(self) -> None:
        """Test case for list_org_members

        """
        pass

    def test_list_public_frontends_for_account(self) -> None:
        """Test case for list_public_frontends_for_account

        """
        pass

    def test_org_account_overview(self) -> None:
        """Test case for org_account_overview

        """
        pass

    def test_overview(self) -> None:
        """Test case for overview

        """
        pass

    def test_version(self) -> None:
        """Test case for version

        """
        pass

    def test_version_inventory(self) -> None:
        """Test case for version_inventory

        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    zrok

    zrok client access

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from zrok_api.models.share_request import ShareRequest

class TestShareRequest(unittest.TestCase):
    """ShareRequest unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ShareRequest:
        """Test ShareRequest
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ShareRequest`
        """
        model = ShareRequest()
        if include_optional:
            return ShareRequest(
                env_zid = '',
                share_mode = 'public',
                frontend_selection = [
                    ''
                    ],
                backend_mode = 'proxy',
                backend_proxy_endpoint = '',
                auth_scheme = '',
                auth_users = [
                    zrok_api.models.auth_user.authUser(
                        username = '', 
                        password = '', )
                    ],
                oauth_provider = '',
                oauth_email_domains = [
                    ''
                    ],
                oauth_authorization_check_interval = '',
                reserved = True,
                permission_mode = 'open',
                access_grants = [
                    ''
                    ],
                unique_name = ''
            )
        else:
            return ShareRequest(
        )
        """

    def testShareRequest(self):
        """Test ShareRequest"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    zrok

    zrok client access

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from zrok_api.api.environment_api import EnvironmentApi


class TestEnvironmentApi(unittest.TestCase):
    """EnvironmentApi unit test stubs"""

    def setUp(self) -> None:
        self.api = EnvironmentApi()

    def tearDown(self) -> None:
        pass

    def test_disable(self) -> None:
        """Test case for disable

        """
        pass

    def test_enable(self) -> None:
        """Test case for enable

        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    zrok

    zrok client access

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from zrok_api.api.admin_api import AdminApi


class TestAdminApi(unittest.TestCase):
    """AdminApi unit test stubs"""

    def setUp(self) -> None:
        self.api = AdminApi()

    def tearDown(self) -> None:
        pass

    def test_add_frontend_grant(self) -> None:
        """Test case for add_frontend_grant

        """
        pass

    def test_add_organization_member(self) -> None:
        """Test case for add_organization_member

        """
        pass

    def test_create_account(self) -> None:
        """Test case for create_account

        """
        pass

    def test_create_frontend(self) -> None:
        """Test case for create_frontend

        """
        pass

    def test_create_identity(self) -> None:
        """Test case for create_identity

        """
        pass

    def test_create_organization(self) -> None:
        """Test case for create_organization

        """
        pass

    def test_delete_account(self) -> None:
        """Test case for delete_account

        """
        pass

    def test_delete_frontend(self) -> None:
        """Test case for delete_frontend

        """
        pass

    def test_delete_frontend_grant(self) -> None:
        """Test case for delete_frontend_grant

        """
        pass

    def test_delete_identity(self) -> None:
        """Test case for delete_identity

        """
        pass

    def test_delete_organization(self) -> None:
        """Test case for delete_organization

        """
        pass

    def test_grants(self) -> None:
        """Test case for grants

        """
        pass

    def test_invite_token_generate(self) -> None:
        """Test case for invite_token_generate

        """
        pass

    def test_list_frontends(self) -> None:
        """Test case for list_frontends

        """
        pass

    def test_list_organization_members(self) -> None:
        """Test case for list_organization_members

        """
        pass

    def test_list_organizations(self) -> None:
        """Test case for list_organizations

        """
        pass

    def test_remove_organization_member(self) -> None:
        """Test case for remove_organization_member

        """
        pass

    def test_update_frontend(self) -> None:
        """Test case for update_frontend

        """
        pass


if __name__ == '__main__':
    unittest.main()



from dataclasses import dataclass, field
from typing import NamedTuple
from .dirs import identityFile, rootDir, configFile, environmentFile, metadataFile
import os
import json
import zrok_api as zrok
from zrok_api.configuration import Configuration
from zrok_api.models.client_version_check_request import ClientVersionCheckRequest

V = "v1.0"


@dataclass
class Metadata:
    V: str = ""
    RootPath: str = ""


@dataclass
class Config:
    ApiEndpoint: str = ""
    DefaultFrontend: str = ""


@dataclass
class Environment:
    Token: str = ""
    ZitiIdentity: str = ""
    ApiEndpoint: str = ""


class ApiEndpoint(NamedTuple):
    endpoint: str
    frm: str


@dataclass
class Root:
    meta: Metadata = field(default_factory=Metadata)
    cfg: Config = field(default_factory=Config)
    env: Environment = field(default_factory=Environment)

    def HasConfig(self) -> bool:
        return self.cfg != Config()

    def Client(self) -> zrok.ApiClient:
        apiEndpoint = self.ApiEndpoint()

        cfg = Configuration()
        cfg.host = apiEndpoint[0] + "/api/v1"

        # Update: Configure authentication token
        # The token needs to be set with 'key' instead of 'x-token'
        # This matches the securityDefinitions in the OpenAPI spec
        cfg.api_key["key"] = self.env.Token

        # Create the API client with the configured authentication
        auth_client = zrok.ApiClient(configuration=cfg)
        self.client_version_check(auth_client)

        return auth_client

    def ApiEndpoint(self) -> ApiEndpoint:
        apiEndpoint = "https://api-v1.zrok.io"
        frm = "binary"

        if self.cfg.ApiEndpoint != "":
            apiEndpoint = self.cfg.ApiEndpoint
            frm = "config"

        env = os.getenv("ZROK_API_ENDPOINT")
        if env != "":
            apiEndpoint = env
            frm = "ZROK_API_ENDPOINT"

        if self.IsEnabled():
            apiEndpoint = self.env.ApiEndpoint
            frm = "env"

        return ApiEndpoint(apiEndpoint.rstrip("/"), frm)

    def IsEnabled(self) -> bool:
        return self.env != Environment()

    def PublicIdentityName(self) -> str:
        return "public"

    def EnvironmentIdentityName(self) -> str:
        return "environment"

    def ZitiIdentityNamed(self, name: str) -> str:
        return identityFile(name)

    def client_version_check(self, zrock_client):
        """Check if the client version is compatible with the API."""
        metadata_api = zrok.MetadataApi(zrock_client)
        try:
            # Perform version check using the client_version_check method
            request = ClientVersionCheckRequest(client_version=V)
            response = metadata_api.client_version_check_with_http_info(
                body=request,
            )

            # Check if the response status code is 200 OK
            if response.status_code != 200:
                raise Exception(f"Client version check failed: Unexpected status code {response.status_code}")

            # Success case - status code is 200 and empty response body is expected
            return

        except Exception as e:
            raise Exception(f"Client version check failed: {str(e)}")


def Default() -> Root:
    r = Root()
    root = rootDir()
    r.meta = Metadata(V=V, RootPath=root)
    return r


def Assert() -> bool:
    exists = __rootExists()
    if exists:
        meta = __loadMetadata()
        return meta.V == V
    return False


def Load() -> Root:
    r = Root()
    if __rootExists():
        r.meta = __loadMetadata()
        r.cfg = __loadConfig()
        r.env = __loadEnvironment()
    else:
        r = Default()
    return r


def __rootExists() -> bool:
    mf = metadataFile()
    return os.path.isfile(mf)


def __assertMetadata():
    pass


def __loadMetadata() -> Metadata:
    mf = metadataFile()
    with open(mf) as f:
        data = json.load(f)
        return Metadata(V=data["v"])


def __loadConfig() -> Config:
    cf = configFile()
    try:
        with open(cf) as f:
            data = json.load(f)
            return Config(
                ApiEndpoint=data.get("api_endpoint", ""),
                DefaultFrontend=data.get("default_frontend", "")
            )
    except (FileNotFoundError, json.JSONDecodeError):
        return Config(ApiEndpoint="", DefaultFrontend="")


def isEnabled() -> bool:
    ef = environmentFile()
    return os.path.isfile(ef)


def __loadEnvironment() -> Environment:
    ef = environmentFile()
    with open(ef) as f:
        data = json.load(f)
        return Environment(
            Token=data["zrok_token"],
            ZitiIdentity=data["ziti_identity"],
            ApiEndpoint=data["api_endpoint"])



from zrok_api.api import ShareApi
from zrok.environment.root import Root
from zrok_api.models.auth_user import AuthUser
from zrok_api.models.share_request import ShareRequest
from zrok_api.models.unshare_request import UnshareRequest
import json
from zrok_api.exceptions import ApiException
import zrok.model as model


class Share():
    root: Root
    request: model.ShareRequest
    share: model.Share

    def __init__(self, root: Root, request: model.ShareRequest):
        self.root = root
        self.request = request

    def __enter__(self) -> model.Share:
        self.share = CreateShare(root=self.root, request=self.request)
        return self.share

    def __exit__(self, exception_type, exception_value, exception_traceback):
        if not self.request.Reserved:
            DeleteShare(root=self.root, shr=self.share)


def CreateShare(root: Root, request: model.ShareRequest) -> model.Share:
    if not root.IsEnabled():
        raise Exception("environment is not enabled; enable with 'zrok enable' first!")

    match request.ShareMode:
        case model.PRIVATE_SHARE_MODE:
            out = __newPrivateShare(root, request)
        case model.PUBLIC_SHARE_MODE:
            out = __newPublicShare(root, request)
        case _:
            raise Exception("unknown share mode " + request.ShareMode)
    out.reserved = request.Reserved
    if request.Reserved:
        out.unique_name = request.UniqueName

    if len(request.BasicAuth) > 0:
        out.auth_scheme = model.AUTH_SCHEME_BASIC
        for pair in request.BasicAuth:
            tokens = pair.split(":")
            if len(tokens) == 2:
                out.auth_users.append(AuthUser(username=tokens[0].strip(), password=tokens[1].strip()))
            else:
                raise Exception("invalid username:password pair: " + pair)

    if request.OauthProvider != "":
        out.auth_scheme = model.AUTH_SCHEME_OAUTH

    try:
        zrok = root.Client()
    except Exception as e:
        raise Exception("error getting zrok client", e)

    try:
        # Use share_with_http_info to get access to the HTTP info and handle custom response format
        share_api = ShareApi(zrok)
        # Add Accept header to handle the custom content type
        custom_headers = {
            'Accept': 'application/json, application/zrok.v1+json'
        }

        response_data = share_api.share_with_http_info(
            body=out,
            _headers=custom_headers
        )

        # Parse response
        if hasattr(response_data, 'data') and response_data.data is not None:
            res = response_data.data
        else:
            raise Exception("invalid response from server")

    except ApiException as e:
        # If it's a content type error, try to parse the raw JSON
        if "Unsupported content type: application/zrok.v1+json" in str(e) and hasattr(e, 'body'):
            try:
                # Parse the response body directly
                res_dict = json.loads(e.body)
                # Create a response object with the expected fields

                class ShareResponse:
                    def __init__(self, share_token, frontend_proxy_endpoints):
                        self.share_token = share_token
                        self.frontend_proxy_endpoints = frontend_proxy_endpoints

                res = ShareResponse(
                    share_token=res_dict.get('shareToken', ''),
                    frontend_proxy_endpoints=res_dict.get('frontendProxyEndpoints', [])
                )
            except (json.JSONDecodeError, ValueError, AttributeError) as json_err:
                raise Exception(f"unable to parse API response: {str(json_err)}") from e
        else:
            raise Exception("unable to create share", e)
    except Exception as e:
        raise Exception("unable to create share", e)

    return model.Share(Token=res.share_token,
                       FrontendEndpoints=res.frontend_proxy_endpoints)


def __newPrivateShare(root: Root, request: model.ShareRequest) -> ShareRequest:
    return ShareRequest(env_zid=root.env.ZitiIdentity,
                        share_mode=request.ShareMode,
                        backend_mode=request.BackendMode,
                        backend_proxy_endpoint=request.Target,
                        auth_scheme=model.AUTH_SCHEME_NONE,
                        permission_mode=request.PermissionMode,
                        access_grants=request.AccessGrants
                        )


def __newPublicShare(root: Root, request: model.ShareRequest) -> ShareRequest:
    ret = ShareRequest(env_zid=root.env.ZitiIdentity,
                       share_mode=request.ShareMode,
                       frontend_selection=request.Frontends,
                       backend_mode=request.BackendMode,
                       backend_proxy_endpoint=request.Target,
                       auth_scheme=model.AUTH_SCHEME_NONE,
                       oauth_email_domains=request.OauthEmailAddressPatterns,
                       oauth_authorization_check_interval=request.OauthAuthorizationCheckInterval,
                       permission_mode=request.PermissionMode,
                       access_grants=request.AccessGrants
                       )
    if request.OauthProvider != "":
        ret.oauth_provider = request.OauthProvider

    return ret


def DeleteShare(root: Root, shr: model.Share):
    req = UnshareRequest(env_zid=root.env.ZitiIdentity,
                         share_token=shr.Token)

    try:
        zrok = root.Client()
    except Exception as e:
        raise Exception("error getting zrok client", e)

    try:
        # Add Accept header to handle the custom content type
        share_api = ShareApi(zrok)
        custom_headers = {
            'Accept': 'application/json, application/zrok.v1+json'
        }

        # Use unshare_with_http_info to get access to the HTTP info
        share_api.unshare_with_http_info(
            body=req,
            _headers=custom_headers
        )
    except ApiException as e:
        # If it's a content type error but the operation was likely successful, don't propagate the error
        if "Unsupported content type: application/zrok.v1+json" in str(e) and (200 <= e.status <= 299):
            # The operation was likely successful despite the content type error
            pass
        else:
            raise Exception("error deleting share", e)
    except Exception as e:
        raise Exception("error deleting share", e)


def ReleaseReservedShare(root: Root, shr: model.Share):
    req = UnshareRequest(env_zid=root.env.ZitiIdentity,
                         share_token=shr.Token,
                         reserved=True)

    try:
        zrok = root.Client()
    except Exception as e:
        raise Exception("error getting zrok client", e)

    try:
        # Add Accept header to handle the custom content type
        share_api = ShareApi(zrok)
        custom_headers = {
            'Accept': 'application/json, application/zrok.v1+json'
        }

        # Use unshare_with_http_info to get access to the HTTP info
        share_api.unshare_with_http_info(
            body=req,
            _headers=custom_headers
        )
    except ApiException as e:
        # If it's a content type error but the operation was likely successful, don't propagate the error
        if "Unsupported content type: application/zrok.v1+json" in str(e) and (200 <= e.status <= 299):
            # The operation was likely successful despite the content type error
            pass
        else:
            raise Exception("error releasing share", e)
    except Exception as e:
        raise Exception("error releasing share", e)
